/* ====================================================================================
   Copyright: AB Circle Ltd
   Description: Simple command line based APDU exchange tool
   Platform: Linux
   ====================================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <winscard.h>

#define CHECK(f, rv) \
    if (SCARD_S_SUCCESS != rv) \
    { \
        printf(f ": %s\n", pcsc_stringify_error(rv)); \
        return -1; \
    }

int main()
{
    SCARDCONTEXT hContext;
    SCARDHANDLE hCard;
    DWORD dwActiveProtocol, dwReaderLen;
    LONG rv;
    char *pcReaders;
    SCARD_READERSTATE rgReaderStates[1];
    BYTE pbRecvBuffer[10];
    DWORD dwRecvLength;

    // Establish context
    rv = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &hContext);
    CHECK("SCardEstablishContext", rv)

    // Get the list of readers
    rv = SCardListReaders(hContext, NULL, NULL, &dwReaderLen);
    CHECK("SCardListReaders", rv)

    pcReaders = calloc(dwReaderLen, sizeof(char));
    rv = SCardListReaders(hContext, NULL, pcReaders, &dwReaderLen);
    CHECK("SCardListReaders", rv)

    // Set up the reader state
    rgReaderStates[0].szReader = pcReaders;
    rgReaderStates[0].dwCurrentState = SCARD_STATE_EMPTY;

    // Wait for a card insertion
    while (rgReaderStates[0].dwEventState & SCARD_STATE_EMPTY)
    {
        rv = SCardGetStatusChange(hContext, 5000, rgReaderStates, 1);
        CHECK("SCardGetStatusChange", rv)
    }

    // Connect to the card
    rv = SCardConnect(hContext, pcReaders, SCARD_SHARE_SHARED, SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1, &hCard, &dwActiveProtocol);
    CHECK("SCardConnect", rv)

    // Set up the APDU
    BYTE pbSendBuffer[] = {0x80, 0x84, 0x00, 0x00, 0x08};
    DWORD dwSendLength = sizeof(pbSendBuffer);

    dwRecvLength = sizeof(pbRecvBuffer);
    rv = SCardTransmit(hCard, SCARD_PCI_T0, pbSendBuffer, dwSendLength, NULL, pbRecvBuffer, &dwRecvLength);
    CHECK("SCardTransmit", rv)

    // Print the result
    for (DWORD i = 0; i < dwRecvLength; i++)
        printf("%02X ", pbRecvBuffer[i]);
    printf("\n");

    // Disconnect and unpower the card
    rv = SCardDisconnect(hCard, SCARD_UNPOWER_CARD);
    CHECK("SCardDisconnect", rv)

    // Release context
    rv = SCardReleaseContext(hContext);
    CHECK("SCardReleaseContext", rv)

    return 0;
}
