/* ====================================================================================
   Copyright: AB Circle Ltd
   Description: Simple command line based Escape command tool
   Platform: Linux
   ====================================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <winscard.h>

#define CHECK(f, rv) \
    if (SCARD_S_SUCCESS != rv) \
    { \
        printf(f ": %s\n", pcsc_stringify_error(rv)); \
        return -1; \
    }

#define SCARD_CTL_CODE(code) (0x42000000 + (code))

// Use Circle CCID driver
#define CIRCLE_CCID

#ifdef CIRCLE_CCID
    // [Using Circle CCID driver]
    #define IOCTL_CCID_ESCAPE SCARD_CTL_CODE(3500)
#else
    /*
        [Using the Linux built-in driver]
        - Open /usr/lib/pcsc/drivers/ccid/Info.plist in edit mode with root priviledge,
        - Locate the line <key>ifdDriverOptions</key>,
        - The following line is typically <string>0000</string>,
        - Define the new value: <string>0001</string>,
        - Save the file and restard pcscd with "systemctl restart pcscd".
    */
    #define IOCTL_CCID_ESCAPE SCARD_CTL_CODE(1)
#endif
 
int main()
{
    SCARDCONTEXT hContext;
    SCARDHANDLE hCard;
    DWORD dwActiveProtocol, dwReaderLen;
    LONG rv;
    char *pcReaders;
    SCARD_READERSTATE rgReaderStates[1];
    BYTE pbRecvBuffer[100];
    DWORD dwRecvLength;

    // Establish context
    rv = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &hContext);
    CHECK("SCardEstablishContext", rv)

    // Get the list of readers
    rv = SCardListReaders(hContext, NULL, NULL, &dwReaderLen);
    CHECK("SCardListReaders", rv)

    pcReaders = calloc(dwReaderLen, sizeof(char));
    rv = SCardListReaders(hContext, NULL, pcReaders, &dwReaderLen);
    CHECK("SCardListReaders", rv)

    // Connect to the reader using Direct mode
    rv = SCardConnect(hContext, pcReaders, SCARD_SHARE_DIRECT, 0, &hCard, &dwActiveProtocol);
    CHECK("SCardConnect", rv)

    // Get firmware version
    BYTE pbSendBuffer[] = {0xFF, 0x00, 0x01, 0x00, 0x00};
    DWORD dwSendLength = sizeof(pbSendBuffer);

    dwRecvLength = sizeof(pbRecvBuffer);
	rv = SCardControl(hCard, IOCTL_CCID_ESCAPE, pbSendBuffer, dwSendLength, pbRecvBuffer, sizeof(pbRecvBuffer), &dwRecvLength);
    CHECK("SCardControl", rv)

    // Print the result (Hex)
    for (DWORD i = 0; i < dwRecvLength; i++)
        printf("%02X ", pbRecvBuffer[i]);
    printf("\n");

    // Print the result (ASCII)

    // Disconnect and unpower the card
    rv = SCardDisconnect(hCard, SCARD_UNPOWER_CARD);
    CHECK("SCardDisconnect", rv)

    // Release context
    rv = SCardReleaseContext(hContext);
    CHECK("SCardReleaseContext", rv)

    return 0;
}